﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;
using System.Management.Instrumentation;
using Microsoft.Win32;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;
using System.Security;
using System.Security.Cryptography;
using System.Windows.Forms;

namespace LANDESK_Web_Remote
{
    public class Shared
    {
        static MainDisplay MainDisplay;
        static ManageRemoteCredentials ManageRemoteCredentials;

        public static void SetMainDisplay(MainDisplay form)
        {
            MainDisplay = form;
        }
        public static MainDisplay GetMainDisplay()
        {
            return MainDisplay;
        }

        public static void SetManageRemoteCredentials(ManageRemoteCredentials form)
        {
            ManageRemoteCredentials = form;
        }
        public static ManageRemoteCredentials GetManageRemoteCredentials()
        {
            return ManageRemoteCredentials;
        }

        // check warranty is a special function that currently works with HP & Dell machines.
        // it will gather needed information and open a page that allows you to easily check the machine's
        // warranty status with minimal effort.

        public static string GetModel()
        {
            string result = string.Empty;

            try
            {
                result = GetDetail("Model", "Win32_ComputerSystem");
            }
            catch { }
            
            return result;
        }
        public static string GetName()
        {
            string result = string.Empty;

            try
            {
                result = (System.Environment.MachineName).ToString();
            }
            catch { }

            return result;
        }
        public static string GetSerialNumber()
        {
            string result = string.Empty;

            try
            {
                result = GetDetail("SerialNumber", "Win32_BIOS");
            }
            catch { }

            return result;
        }
        public static string GetCurrentUser()
        {
            string result = string.Empty;

            try
            {
                result = Environment.UserName.ToString();
                //result = System.Security.Principal.WindowsIdentity.GetCurrent().Name.ToString();
            }
            catch { }

            return result;
        }
        public static string GetIP()
        {
            string result = string.Empty;

            try
            {
                /*
                    This code below works fine, but provides multiple IPv6 addresses.
                    I could compare results, but the solution used below guarentees IPv4 gets
                    returned. The code below will be kept if alternative is ever needed.
                 
                    string host = Dns.GetHostName();
                    IPHostEntry ip = Dns.GetHostEntry(host);
                    result = ip.AddressList[2].ToString();
                 */

                IPHostEntry host;
                host = Dns.GetHostEntry(Dns.GetHostName());
                foreach (IPAddress ip in host.AddressList)
                {
                    if (ip.AddressFamily == AddressFamily.InterNetwork)
                    {
                        result = ip.ToString();
                        break;
                    }
                }
                
            }
            catch { }

            return result;
        }
        public static string GetOperatingSystem()
        {
            string result = string.Empty;

            try
            {
                result = GetDetail("Caption", "Win32_OperatingSystem");
                if (Environment.Is64BitOperatingSystem)
                {
                    result = result + " (64-bit)";
                }
                else
                {
                    result = result + " (32-bit)";
                }
                result = result.Remove(0, 10);
            }
            catch { }

            return result;
        }
        public static string GetProductNumber()
        {
            string result = string.Empty;

            try // to get the product number from the registry
            {
                const string userRoot = "HKEY_LOCAL_MACHINE";
                const string subkey = "HARDWARE\\DESCRIPTION\\System\\BIOS";
                const string keyName = userRoot + "\\" + subkey;
                result = Registry.GetValue(keyName, "SystemSKU", "").ToString();
            }
            catch { }

            string vendor = GetVendor();

            if ((vendor == "Hewlett-Packard") || (vendor == "HP"))
            {
                if (result.Contains('#'))
                {
                    result = result.Substring(0, result.Length - 4);
                }
            }


            return result;
        }

        public static string GetSCMCode()
        {
            string result = string.Empty;

            try
            {
                result = System.IO.File.ReadAllText(@"C:\scm\scmcode.txt");
            }
            catch { }

            return result;
        }
        public static string GetUptime()
        {
            string result = string.Empty;

            try
            {
                result = GetDetail("LastBootUpTime", "Win32_OperatingSystem");
                DateTime lastReboot = ManagementDateTimeConverter.ToDateTime(result);
                result = (DateTime.Now.ToUniversalTime() - lastReboot.ToUniversalTime()).ToString();
                result = result.Remove((result.Length - 8), 8); // remove miliseconds.
                // If days have gone by, call them days.
                if (result.Length>8)
                {
                    string hms = result.Substring((result.Length - 8), 8);
                    string days = result.Remove((result.Length - 9), 9)+" Days, ";
                    result = days + hms;
                }
                
            }
            catch { }

            return result;
        }
        public static string GetVendor()
        {
            string result = string.Empty;

            try
            {
                result = GetDetail("Manufacturer", "Win32_BIOS");
            }
            catch { }

            return result;
        }
        // TEMP NOTE: I changed BIOS -> PROCESS
        public static string GetDetail(string requestedDetail, string detailLocation)
        {
            string result = string.Empty;

            try
            {
                ManagementObjectSearcher searcher = new ManagementObjectSearcher("SELECT "+requestedDetail+" FROM " + detailLocation);
                foreach (ManagementObject process in searcher.Get())
                {
                    result = process[requestedDetail].ToString();
                    break;
                }
            }
            catch { }

            return result;
        }

        static byte[] entropy = System.Text.Encoding.Unicode.GetBytes("Salt Is Not A Password");

        public static string EncryptString(System.Security.SecureString input)
        {
            byte[] encryptedData = ProtectedData.Protect(
                System.Text.Encoding.Unicode.GetBytes(ToInsecureString(input)),
                entropy,
                System.Security.Cryptography.DataProtectionScope.CurrentUser);
            return Convert.ToBase64String(encryptedData);
        }

        public static SecureString DecryptString(string encryptedData)
        {
            try
            {
                byte[] decryptedData = System.Security.Cryptography.ProtectedData.Unprotect(
                    Convert.FromBase64String(encryptedData),
                    entropy,
                    System.Security.Cryptography.DataProtectionScope.CurrentUser);
                return ToSecureString(System.Text.Encoding.Unicode.GetString(decryptedData));
            }
            catch
            {
                return new SecureString();
            }
        }

        public static SecureString ToSecureString(string input)
        {
            SecureString secure = new SecureString();
            foreach (char c in input)
            {
                secure.AppendChar(c);
            }
            secure.MakeReadOnly();
            return secure;
        }

        public static string ToInsecureString(SecureString input)
        {
            string returnValue = string.Empty;
            IntPtr ptr = System.Runtime.InteropServices.Marshal.SecureStringToBSTR(input);
            try
            {
                returnValue = System.Runtime.InteropServices.Marshal.PtrToStringBSTR(ptr);
            }
            finally
            {
                System.Runtime.InteropServices.Marshal.ZeroFreeBSTR(ptr);
            }
            return returnValue;
        }

        public static IPAddress GetIpFromHost(string host)
        {
            //variable to hold our error message (if something fails)
            string errMessage = string.Empty;
 
            //IPAddress instance for holding the returned host
            IPAddress address = null;
 
            //wrap the attempt in a try..catch to capture
            //any exceptions that may occur
            try
            {
                //get the host IP from the name provided
                address = Dns.GetHostEntry(host).AddressList[0];
            }
            catch (SocketException ex)
            {
                //some DNS error happened, return the message
                errMessage = string.Format("DNS Error: {0}", ex.Message);
            }
            return address;
        }

        public static String GetWinPath()
        {
            return Environment.GetFolderPath(Environment.SpecialFolder.Windows);
        }

    }
}
